import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

/**
 * The following demo application shows how to open a connection 
 * to a PostgreSQL database, create a prepared statement, fill
 * in the variables, and how to properly close the connection to the db.
 * 
 * Um lokal entwickeln zu knnen ohne eine DB installiern zu mssen, bietet sich
 * an, einen SSH Tunnel zum bungsserver aufzubauen:
 * > ssh -L 5432:localhost:5432 bordo.dbai.tuwien.ac.at -l IHRLOGIN
 * danach kann die DB angesprochen werden als ob sie lokal installiert ist.
 */
public final class Application {
	
	/**
	 * Configure the connection and open it.
	 * Make sure to properly close the connection.
	 */
	public static void main(String[] args) {
		// Configure the connection properties
		Connection conn = null;
		
		try {
		   Class.forName("org.postgresql.Driver");
		   conn = DriverManager.getConnection("jdbc:postgresql:<<database>>","<<user>>","<<pass>>");
		   programLogic(conn);
		} catch (ClassNotFoundException e) {
		    e.printStackTrace(); 
		} catch (SQLException e) {
			/* do some error handling, for debug purposes a stacktrace is useful */
			e.printStackTrace();
		} finally {
			closeConnection(conn);
		}
	}

	/**
	 * Use the connection to select some data from the database and print it
	 * on the console.
	 * 
	 * Demonstrates the use of a prepared statement.
	 * 
	 * Make sure the statement and result sets are properly closed.
	 */
	private static void programLogic(Connection conn) {
		PreparedStatement preparedStmt = null;
		try {
			for(int semester = 1; semester <= 10; ++semester) {
				System.out.println("Students for Semester "+semester);
				
				preparedStmt = conn.prepareStatement("SELECT matrnr, name FROM students WHERE semester = ?");
				preparedStmt.setInt(1, semester);
				
				// create and open a result set
				ResultSet rs = null;
				try {
					rs = preparedStmt.executeQuery();
					while (rs.next()) {
						System.out.print("  ");
						System.out.print(rs.getInt(1)); // alternatively rs.getInt("matrnr")
						System.out.print(" - ");
						System.out.println(rs.getString(2)); // alternatively rs.getString("name")
					}
				} catch (SQLException e) {
					/* do some error handling, for debug purposes a stacktrace is useful */
					e.printStackTrace();
				} finally {
					closeResultSet(rs);
				}
			}
		} catch(SQLException e) {
			/* do some error handling, for debug purposes a stacktrace is useful */
			e.printStackTrace();
		} finally {
			closeStatement(preparedStmt);
		}
	}

/* some helpers */
	
	private static void closeConnection(Connection conn) {
		if (conn != null) { try { conn.close();	} catch (SQLException e) {/*...*/} }
	}

	private static void closeStatement(PreparedStatement preparedStmt) {
		if (preparedStmt != null) {
			try {
				preparedStmt.close();
			} catch (SQLException e) {/*...*/}
		}
	}

	private static void closeResultSet(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {/*...*/}
		}
	}
}
